@@ -15,10 +15,18 @@ INPUT_DTYPE=np.float
 TARGET_DTYPE=np.int64
 DEFAULT_SIZE=256
 DEFAULT_OVERLAP=0
-TO_CATEGORICAL_ERROR='imagebox.handler: nb_categories required for to_categorical'
 INPUT_RESAMPLING=Resampling.bilinear
 TARGET_RESAMPLING=Resampling.mode
-
+TO_CATEGORICAL_ERROR=(
+    'imagebox.handler: '
+    'nb_categories required for to_categorical' )
+SAFE_RESCALE_ERROR=(
+    'imagebox.handler: '
+    'target rescale leads to fractional value. '
+    'use safe_rescale=False to force' )
+DIMS_REQUIRED_ERROR=(
+    'imagebox.handler: '
+    'width and height, or example_path, required for (float)cropping' )
 
 #
 # InputTargetHandler
@@ -104,9 +112,11 @@ class InputTargetHandler(object):
             input_padding_value=0,
             target_padding_value=0,
             float_cropping=None,           
+            target_ratio=1,
+            safe_rescale=True,
             cropping=None,
             input_cropping=None,
-            target_cropping=None,
+            target_cropping='auto',
             window_index=None,
             size=None,
             width=None,
@@ -148,19 +158,18 @@ class InputTargetHandler(object):
         self.target_padding=target_padding or padding
         self.input_padding_value=input_padding_value
         self.target_padding_value=target_padding_value
-        self.cropping=cropping or 0
-        self.input_cropping=input_cropping
-        self.target_cropping=target_cropping
-        self.float_cropping=float_cropping
-        if size:
-            self.width=size
-            self.height=size
-        else:
-            self.width=width
-            self.height=height
-        self.set_window(
-            window_index=window_index,
-            example_path=example_path)
+        self._set_cropping_and_dims(
+            cropping,
+            target_ratio,
+            safe_rescale,
+            input_cropping,
+            target_cropping,
+            float_cropping,
+            size,
+            width,
+            height,
+            example_path)
+        self.set_window(window_index=window_index)
         self.target_expand_axis=target_expand_axis
         self.input_preprocess=input_preprocess
         self.target_preprocess=target_preprocess
@@ -175,7 +184,7 @@ class InputTargetHandler(object):
             path,
             self.input_resolution,
             self.target_resampling,
-            window )
+            window or self.input_window )
         im=process_input(
             im,
             preprocess=self.input_preprocess,
@@ -185,7 +194,6 @@ class InputTargetHandler(object):
             indices_dict=self.indices_dict,
             padding=self.input_padding,
             padding_value=self.input_padding_value,
-            cropping=self.input_cropping,
             bounds=self.input_bounds,
             means=self.means,
             stdevs=self.stdevs,
@@ -202,7 +210,7 @@ class InputTargetHandler(object):
             path,
             self.target_resolution,
             self.target_resampling,
-            window )
+            window or self.target_window )
         im=process_target(
             im,
             preprocess=self.target_preprocess,
@@ -213,7 +221,6 @@ class InputTargetHandler(object):
             nb_categories=self.nb_categories,
             padding=self.target_padding,
             padding_value=self.target_padding_value,
-            cropping=self.target_cropping,
             expand_axis=self.target_expand_axis,
             squeeze=self.target_squeeze,
             dtype=self.target_dtype )
@@ -232,65 +239,118 @@ class InputTargetHandler(object):
     
 
     def set_window(self,window_index=None,example_path=None):
-        self._ensure_dimensions(example_path)
         if self.tiller:
             if window_index is None:
                 window_index=randint(0,len(self.tiller)-1)
             self.window_index=window_index            
-            window=self.tiller[self.window_index]
+            target_window=self.tiller[self.window_index]
+            input_window=target_window
         else:
             self.window_index=False
-            window=None
-        if self.cropping:
-            window=self._shift_crop_window(
-                window,
-                dx=self.cropping,
-                dy=self.cropping,
-                crop=self.cropping)
+            input_window=0,0,self.input_width,self.input_height
+            target_window=0,0,self.target_width,self.target_height
+        self.input_window=self._shift_crop_window(
+                input_window,
+                dx=self.input_cropping,
+                dy=self.input_cropping,
+                crop=self.input_cropping)
+        self.target_window=self._shift_crop_window(
+                target_window,
+                dx=self.target_cropping,
+                dy=self.target_cropping,
+                crop=self.target_cropping)
         if self.float_cropping:
-            self.float_x=randint(0,2*self.float_cropping)
-            self.float_y=randint(0,2*self.float_cropping)
-            window=self._shift_crop_window(
-                window,
-                dx=self.float_x,
-                dy=self.float_y,
-                crop=self.float_cropping)
-        self.window=window
+            self.float_x=self._random_delta()
+            self.float_y=self._random_delta()            
+            self.input_window=self._shift_crop_window(
+                    self.input_window,
+                    dx=self.float_x,
+                    dy=self.float_y,
+                    crop=self.float_cropping )
+            tfc=self._target_rescale(self.float_cropping)
+            self.target_window=self._shift_crop_window(
+                    self.target_window,
+                    dx=self._target_rescale(self.float_x),
+                    dy=self._target_rescale(self.float_y),
+                    crop=self._target_rescale(self.float_cropping) )
+
 
     
     #
     # INTERNAL METHODS
     #
+    def _set_cropping_and_dims(self,
+            cropping,
+            target_ratio,
+            safe_rescale,
+            input_cropping,
+            target_cropping,
+            float_cropping,
+            size,
+            width,
+            height,
+            example_path):
+        self.target_ratio=target_ratio or 1
+        self.input_cropping=input_cropping or cropping or 0
+        self.safe_rescale=safe_rescale
+        if size:
+            self.input_width=size
+            self.input_height=size
+        else:
+            self.input_width=width
+            self.input_height=height
+        self._ensure_dimensions(example_path)
+        self.target_width=self._target_rescale(self.input_width)
+        self.target_height=self._target_rescale(self.input_height)
+        if self.input_cropping and (target_cropping=='auto'):
+            self.target_cropping=self._target_rescale(self.input_cropping)
+        else:
+            self.target_cropping=target_cropping
+        self.float_cropping=float_cropping
+
+
+    def _target_rescale(self,value):
+        fval=value*self.target_ratio
+        val=round(fval)
+        if (not self.safe_rescale) or (val==fval):
+            return val
+        else:
+            raise ValueError(SAFE_RESCALE_ERROR)
+
+
     def _read(self,path,resolution,resampling,window):
         return io.read(
             path,
-            window=window or self.window,
+            window=window,
             res=resolution,
             resampling=resampling)
 
 
-    def _shift_crop_window(self,window=None,dx=0,dy=0,crop=0):
-        if not window:
-            window=(0,0,self.width,self.height)
-        x=window[0]+dx
-        y=window[1]+dy
-        w=window[2]-2*crop
-        h=window[3]-2*crop
-        return x,y,w,h
+    def _random_delta(self):
+        return randint(0,2*int(self.float_cropping*self.target_ratio))
+
+
+    def _shift_crop_window(self,window,dx=0,dy=0,crop=0):
+        if dx or dy or crop:
+            x=window[0]+dx
+            y=window[1]+dy
+            w=window[2]-2*crop
+            h=window[3]-2*crop
+            return x,y,w,h
+        else:
+            return window
 
 
     def _ensure_dimensions(self,example_path):
-        if not (self.width and self.height):
-            self.float_x=False
-            self.float_y=False
+        if not (self.input_width and self.input_height):
             if example_path:
                 shape=io.read(example_path,return_profile=False).shape
                 if BANDS_FIRST:
-                    self.height,self.width=shape[1:]
+                    self.input_height,self.input_width=shape[1:]
                 else:
-                    self.height,self.width=shape[:2]
-            elif self.cropping or self.float_cropping:
-                raise ValueError('io.handler: width and height, or example_path, required for (float)cropping')
+                    self.input_height,self.input_width=shape[:2]
+            elif self.input_cropping or self.target_cropping or self.float_cropping:
+                raise ValueError(DIMS_REQUIRED_ERROR)
 
 
     def _return_data(self,im,profile,return_profile):
